<?php

namespace Drupal\edlp_studio\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\User\UserDataInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\Core\Url;

use Drupal\edlp_studio\Entity\Compositon;

/**
 * Class CompositionController.
 */
class CompositionController extends ControllerBase {

  protected $user;
  protected $userdata;

  /**
   * Class constructor.
   */
  public function __construct(AccountInterface $account, UserDataInterface $userdata) {
    $this->user = $account;
    $this->userdata = $userdata;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    return new static(
      // Load the service required to construct this class.
      $container->get('current_user'),
      $container->get('user.data')
    );
  }


  /**
   * AdddContent.
   *
   * @return json
   *   Return status.
   */
  public function CompositionActionJson($action, $cid, Request $request) {
    $this->error_message = null;
    $status = 'ok';
    $message = 'Hello';

    switch($action){
      case 'create':
        $name = $request->query->get('new_name');
        if($name){
          $this->createComposition($name);
        }else{
          $this->error_message = t("Composition creation needs a name as query paramater!");
        }
        break;
      case 'open':
        if($cid){
          $this->openComposition($cid);
        }else{
          $this->error_message = t("Composition opening needs a cid as url paramater!");
        }
        break;
      case 'delete':
        if($cid){
          $this->deleteComposition($cid);
        }else{
          $this->error_message = t("Composition deletion needs a cid as url paramater!");
        }
        break;
      // case 'rename':
      //
      //   break;
    }

    if($this->error_message){
      $status = 'error';
      $message = $this->error_message;
    }

    // JSON
    $response = new JsonResponse();
    $data = array(
      'action' => $action,
      'status' => $status,
      'message' => $this->error_message
    );
    if($status == 'ok'){
      switch ($action) {
        case 'create':
          $url = Url::fromRoute('entity.composition.canonical', ['composition' => $this->compo->id()], ['absolute' => TRUE]);
          $title = $this->compo->getName();
          $new_link_build = array(
            '#title' => $title,
            '#type' => 'link',
            '#url' => $url,
            '#options'=>array(
              'attributes' => array(
                'data-drupal-link-system-path' => $url->getInternalPath(),
                'nid' => $this->compo->id(),
                'class' => ['composition-link'],
                'title'=>$title,
              ),
            ),
          );

          $data += array(
            'new_name' => $title,
            'new_link' => render($new_link_build),
          );
          break;
        case 'open':
          $this->getRendredComposition();
          $data += array(
            'compo' => $this->rendered_compo,
          );
          break;
        // case 'delete':
        //   # code...
        //   break;
      }
    }

    $response->setData($data);
    return $response;

    // classic html
    // return array(
    //   '#markup'=>'Status : ' . $status . ' | Message : ' . $message,
    // );
  }

  private function createComposition($name){
    $compo = array(
      'id' => NULL,
      'uid' => $this->user->id(),
      'status' => TRUE,
      'name' => $name,
    );
    $this->compo = entity_create('composition', $compo);
    $this->compo->save();
  }

  private function getRendredComposition(){
    $view_builder = \Drupal::entityTypeManager()->getViewBuilder('composition');
    $compobuild = $view_builder->view($this->compo, 'default');
    $this->rendered_compo = render($compobuild);
  }

  private function openComposition($cid){
    $this->compo = entity_load('composition', $cid);
  }

  private function deleteComposition($cid){
    $this->compo = entity_load('composition', $cid);
    $this->compo->delete();
  }
}
